/*_############################################################################
  _## 
  _##  SNMP4J-Agent 2 - MOScopeComparator.java  
  _## 
  _##  Copyright (C) 2005-2011  Frank Fock (SNMP4J.org)
  _##  
  _##  Licensed under the Apache License, Version 2.0 (the "License");
  _##  you may not use this file except in compliance with the License.
  _##  You may obtain a copy of the License at
  _##  
  _##      http://www.apache.org/licenses/LICENSE-2.0
  _##  
  _##  Unless required by applicable law or agreed to in writing, software
  _##  distributed under the License is distributed on an "AS IS" BASIS,
  _##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  _##  See the License for the specific language governing permissions and
  _##  limitations under the License.
  _##  
  _##########################################################################*/


package org.snmp4j.agent;

import java.util.Comparator;
import org.snmp4j.smi.OctetString;

/**
 * The <code>MOScopeComparator</code> compares two scopes with each other.
 * <p>
 * Two scopes are compared by their context (if both are {@link MOContextScope}
 * instances) first and then by their lower bound.
 * <p>
 * A scope is compared with a query by comparing the scope with the queries
 * scope and then if both are deemed to be equal, the upper bound of the scope
 * is checked. If it is unbounded (upper bound is <code>null</code), then
 * the scoped is deemed to be greater than the query. Otherwise, the upper bound
 * of the scope is compared with the lower bound of the query. Scope and query
 * are deemd to be equal if both bounds are equal and are both included.
 * Otherwise the scope is deemed to be less than the query.
 *
 * @author Frank Fock
 * @version 1.0
 */
public class MOScopeComparator implements Comparator<MOScope> {

  public MOScopeComparator() {
  }

  /**
   * Compares a scope with another scope or query. See also the class
   * description how comparison is done.
   * @param o1
   *   a MOscope instance.
   * @param o2
   *   a MOscope instance.
   * @return
   *   an integer less than zero if <code>o1</code> is less than <code>o2</code>
   *   and zero if both values are deemed to be equal and a value greater than
   *   zero if <code>o1</code> is greater than <code>o2</code>.
   */
  public int compare(MOScope o1, MOScope o2) {
    if (o1 == o2) {
      return 0; // ensure identity is equal
    }
    int result = 0;
    if (o2 instanceof MOContextScope) {
      result = compareContextScope(o1, (MOContextScope) o2);
    }
    else if (o1 instanceof MOContextScope) {
      result = -compareContextScope(o2, (MOContextScope) o1);
    }
    else {
      result = o1.getLowerBound().compareTo(o2.getLowerBound());
      if (result == 0) {
        if ((!o1.isUpperIncluded()) ||
            (!o2.isLowerIncluded())) {
          return -1;
        }
      }
    }
    return result;
  }

  private static int compareContextScope(MOScope scope, MOContextScope scope2) {
    int result = 0;
    if (scope instanceof MOContextScope) {
      OctetString c1 = ((MOContextScope)scope).getContext();
      OctetString c2 = scope2.getContext();
      if ((c1 != null) && (c2 != null)) {
        result = c1.compareTo(c2);
      }
    }
    if (result != 0) {
      return result;
    }
    if (scope.getUpperBound() == null) {
      return 1;
    }
    else {
      result =
          scope.getUpperBound().compareTo(scope2.getLowerBound());
      if (result == 0) {
        if ((!scope.isUpperIncluded()) ||
            (!scope2.isLowerIncluded())) {
          return -1;
        }
      }
    }
    return result;
  }

  public boolean equals(Object obj) {
    if (obj instanceof MOScopeComparator) {
      return (this == obj);
    }
    return false;
  }

  public int hashCode() {
    return super.hashCode();
  }
}
